/* Copyright 2024 EPFL, Lausanne */

package boardgame

/** To run this test suite, start "sbt" then run the "test" command. */
class BoardGameSuite extends munit.FunSuite:

  import defs.*
  import BoardGame.*
  import BoardGameSuiteIO.*

  extension (t: Tile)
    def desc: String =
      val base = t.base match
        case TileBase.FlatTerrain(base) => s"flat terrain (${base.toString.toLowerCase})"
        case TileBase.HillTerrain(base) => s"hills (${base.toString.toLowerCase})"
        case other                      => other.toString.toLowerCase
      val feature = t.feature match
        case Some(Feature.RainForest)        => Seq("rain forest")
        case Some(Feature.GeothermalFissure) => Seq("geothermal fissure")
        case Some(other)                     => Seq(other.toString.toLowerCase)
        case None()                          => Seq.empty
      val resource = t.resource match
        case Some(r) => Seq(r.toString.toLowerCase)
        case None()  => Seq.empty
      val const = t.construction match
        case Some(Construction.City(_)) => Seq("city")
        case Some(Construction.District(kind)) =>
          Seq(kind match
            case DistrictKind.Campus         => "campus"
            case DistrictKind.IndustrialZone => "industrial zone"
            case DistrictKind.CommercialHub  => "commercial hub"
          )
        case Some(Construction.Exploitation(kind)) =>
          Seq(kind match
            case ResourceImprovement.FishingBoats => "fishing boats"
            case ResourceImprovement.LumberMill   => "lumber mill"
            case other                            => other.toString.toLowerCase
          )
        case None() => Seq.empty
      (Seq(base) ++ feature ++ resource ++ const).mkString(", ")

  extension (t: Tests)
    def title: String =
      t match
        case WarmupTest(x, y, _, pts) =>
          s"tile indexing at ($x, $y) ($pts pts)"
        case Task1Test(tile, _, pts) =>
          s"tile yields for ${tile.desc} ($pts pts)"
        case Task2Test(x, y, _, pts) =>
          s"adjacency bonus at ($x, $y) ($pts pts)"
        case Task3aTest(x, y, radius, _, pts) =>
          s"tiles at ring $radius from ($x, $y) ($pts pts)"
        case Task3bTest(x, y, radius, _, pts) =>
          s"all tiles within $radius from ($x, $y) ($pts pts)"
        case Task3Test(_, x, y, _, pts) =>
          s"city settlement at ($x, $y) ($pts pts)"

    def execute: Unit =
      t match
        case WarmupTest(x, y, exp, _) =>
          val got = warmup.wm.tile(x, y)
          assertEquals(got, exp, s"tile indexing is incorrect")
        case Task1Test(tile, exp, _) =>
          val got = tile.yields
          assertEquals(got, exp, s"yields calculation incorrect")
        case Task2Test(x, y, exp, _) =>
          val got = adjacencyBonus(task2.wm, x, y)
          assertEquals(got, exp, s"adjacency bonus calculation incorrect")
        case Task3aTest(x, y, radius, exp, _) =>
          val got = collectTilesInRing(task3ab.wm, x, y, radius)
          assertEquals(got, exp, "collecting tiles at ring incorrect")
        case Task3bTest(x, y, radius, exp, _) =>
          val got = allTilesWithinRadius(task3ab.wm, x, y, radius)
          assertEquals(got, exp, "collecting all tiles within ring incorrect")
        case Task3Test(wm, x, y, exp, _) =>
          val got = validCitySettlement(wm, x, y)
          assertEquals(got, exp, "city settlement validity incorrect")

  allTests.foreach(t =>
    test(t.title) { t.execute }
  )

  override val munitTimeout = scala.concurrent.duration.Duration(1, "s")
